import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.Enumeration;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;

/**
 * Ein grafisches Interface in Swing zum SimpleConfigParser.
 *
 * @author  <a href="mailto:jochen@kupperschmidt.de">Jochen Kupperschmidt</a>
 * @version 15. November 2002
 */
public class SwingUI implements ActionListener {
    private static final boolean DEBUG       = true;
    private static final String  APP_NAME    = "SimpleConfigParser";

    private static final String  ICON_OPEN   = "icons/Open16.gif";
    private static final String  ICON_SAVE   = "icons/Save16.gif";
    private static final String  ICON_EDIT   = "icons/Edit16.gif";
    private static final String  ICON_ABOUT  = "icons/About16.gif";
    private static final String  ICON_EXIT   = "icons/Stop16.gif";

    private static final String  LABEL_OPEN  = "\u00d6ffnen";
    private static final String  LABEL_SAVE  = "Speichern";
    private static final String  LABEL_EDIT  = "Bearbeiten";
    private static final String  LABEL_ABOUT = "\u00dcber";
    private static final String  LABEL_EXIT  = "Beenden";

    private String               filename;
    private SimpleConfigParser   scp;

    private JFrame               frame       = new JFrame();
    private JPanel               contentPane = new JPanel();
    private JToolBar             toolBar     = new JToolBar();
    private JLabel               statusBar   = new JLabel(" ");

    private JTextArea            configField;
    private JPanel               mainPanel;
    private JPanel               showPanel;
    private JPanel               editPanel;
    private JTextArea            textArea;

    /* Einsprungspunkt */
    public static void main(String[] args) {
        SwingUI ui = new SwingUI();
    }

    /* Konstruktor */
    public SwingUI() {
        // Programm soll beim Schliessen des Fensters beendet werden.
        frame.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    System.exit(0);
                }
            }
        );

        // Hauptfenster zusammensetzen
        contentPane.setLayout(new BorderLayout(2, 4));
        frame.setJMenuBar(buildMenuBar());
        buildToolBar();
        toolBar.setOrientation(JToolBar.VERTICAL);
        contentPane.add(toolBar, BorderLayout.WEST);

        configField = new JTextArea("nicht geladen");
        configField.setLineWrap(false);
        configField.setEditable(false);

        showPanel = new JPanel();
        showPanel.setBorder(BorderFactory.createTitledBorder(
                BorderFactory.createEtchedBorder(), "Konfiguration"));
        showPanel.setLayout(new BorderLayout());
        showPanel.add(new JScrollPane(configField), BorderLayout.CENTER);

        contentPane.add(showPanel, BorderLayout.CENTER);
        contentPane.add(statusBar, BorderLayout.SOUTH);

        frame.getContentPane().add(contentPane);
        frame.setTitle(APP_NAME);
        frame.setSize(400, 280);
        frame.setLocation(240, 120);
        frame.setVisible(true);
    }

    /* Event-Handler */
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        debug("ActionCommand: " + cmd);
        if (cmd.equals("EXIT")) {
            // Programm beenden
            System.exit(0);
        } else if (cmd.equals("OPEN")) {
            // Konfiguration laden
            debug("Loading config.");

            JFileChooser fc = new JFileChooser();
            fc.showOpenDialog(frame);
            try {
                filename = fc.getSelectedFile().getName();
                statusBar.setText(filename + " ge\u00f6ffnet.");
            } catch (NullPointerException e) {
                return;
            }
            try {
                scp = new SimpleConfigParser(filename);
            } catch (IOException e) {
                errorBox("Fehler beim Laden der Konfiguration:\n"
                        + e.getMessage());
                return;
            }

            showConfig();
        } else if (cmd.equals("SAVE")) {
            // Konfiguration speichern
            debug("Saving config.");

            statusBar.setText(filename + " gespeichert.");
            try {
                scp.saveConfig();
            } catch (IOException e) {
                errorBox("Fehler beim Speichern der Konfiguration:\n"
                        + e.getMessage());
            }
        } else if (cmd.equals("EDIT")) {
            // Konfiguration ndern
            debug("Editing config.");
            try {
                int index = getPairToEdit();
                debug("Selected item: " + index);
                KeyValuePair editPair = (KeyValuePair) scp.config.elementAt(index);
                SwingUIDialog dialog = new SwingUIDialog(frame, editPair);
                try {
                    scp.editConfig(index, editPair.getKey(), editPair.getValue());
                } catch (Exception e) {
                    errorBox("Fehler:\n" + e.getMessage());
                    return;
                }
                statusBar.setText("Konfiguration ge\u00e4ndert.");
                showConfig();
            } catch (NullPointerException ignored) {}
        } else if (cmd.equals("TOOLBAR")) {
            // zeigt/versteckt die Symbolleiste
            toolBar.setVisible(!toolBar.isVisible());
        } else if (cmd.equals("STATUSBAR")) {
            // zeigt/versteckt die Statusleiste
            statusBar.setVisible(!statusBar.isVisible());
        } else if (cmd.equals("ABOUT")) {
            // gibt Programm-Hinweis aus
            String aboutText = "Danke, dass Sie dieses Programm benutzen!\n";
            aboutText       += " Copyright (c) 2002 Jochen Kupperschmidt \n";
            JOptionPane.showMessageDialog(frame, aboutText, LABEL_ABOUT,
                    JOptionPane.PLAIN_MESSAGE);
        }
    }

    /* zeigt die Daten der geladenen Konfiguration an */
    private void showConfig() {
        frame.setTitle(APP_NAME + " <" + filename + ">");
        configField.setText("");
        int idx = 0;
        for (Enumeration el = scp.config.elements(); el.hasMoreElements(); ) {
            KeyValuePair pair = (KeyValuePair) el.nextElement();
            if (pair != null) {
                configField.append("#:" + idx + "\t");
                configField.append(pair.getKey() + "\t=    ");
                configField.append(pair.getValue() + "\t");
                configField.append("[" + pair.getType() + "]\n");
            }
            idx++;
        }
    }

    /* fragt vom Benutzer das zu bearbeitende Paar ab */
    private int getPairToEdit() throws NullPointerException {
        // Anlegen eines Arrays mit den whlbaren Schlsseln
        String[] keys = new String[scp.config.size()];
        int idx = 0;
        for (Enumeration el = scp.config.elements(); el.hasMoreElements(); ) {
            KeyValuePair pair = (KeyValuePair) el.nextElement();
            if (pair != null) {
                keys[idx] = "#" + idx + ":  " + pair.getKey();
            }
            idx++;
        }

        // Abfragen des zu bearbeitenden Schlssel-Wert-Paares
        String sel = (String) JOptionPane.showInputDialog(frame,
                "zu bearbeitender Eintrag:", "Eintrag bearbeiten",
                JOptionPane.QUESTION_MESSAGE, null, keys, keys[0]);
        sel = sel.substring(1, sel.indexOf(':'));
        return (new Integer(sel)).intValue();
    }

    /* erzeugt das Men */
    private JMenuBar buildMenuBar() {
        JMenuBar    menuBar = new JMenuBar();
        JMenu       menu;
        JMenuItem   menuItem;

        // Men 'Datei'
        menu = new JMenu("Datei");
        menuBar.add(menu);

        // Menpunkt 'ffnen'
        menuItem = new JMenuItem(LABEL_OPEN);
        menuItem.setIcon(new ImageIcon(ICON_OPEN));
        menuItem.setActionCommand("OPEN");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        // Menpunkt 'Speichern'
        menuItem = new JMenuItem(LABEL_SAVE);
        menuItem.setIcon(new ImageIcon(ICON_SAVE));
        menuItem.setActionCommand("SAVE");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu.addSeparator();

        // Menpunkt 'Beenden'
        menuItem = new JMenuItem(LABEL_EXIT);
        menuItem.setIcon(new ImageIcon(ICON_EXIT));
        menuItem.setActionCommand("EXIT");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        // Men 'Bearbeiten'
        menu = new JMenu("Bearbeiten");
        menuBar.add(menu);

        // Menpunkt 'Bearbeiten'
        menuItem = new JMenuItem("Konfiguration bearbeiten");
        menuItem.setActionCommand("EDIT");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        // Men 'Ansicht'
        menu = new JMenu("Ansicht");
        menuBar.add(menu);

        // Menpunkt 'Symbolleiste'
        menuItem = new JCheckBoxMenuItem("Symbolleiste", true);
        menuItem.setActionCommand("TOOLBAR");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        // Menpunkt 'Statusleiste'
        menuItem = new JCheckBoxMenuItem("Statusleiste", true);
        menuItem.setActionCommand("STATUSBAR");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuBar.add(Box.createHorizontalGlue());

        // Men 'Hilfe'
        menu = new JMenu("Hilfe");
        menuBar.add(menu);

        // Menpunkt 'ber'
        menuItem = new JMenuItem(LABEL_ABOUT);
        menuItem.setIcon(new ImageIcon(ICON_ABOUT));
        menuItem.setActionCommand("ABOUT");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        return menuBar;
    }

    /* erzeugt die Toolbar */
    private void buildToolBar() {
        JButton button = null;

        // Toolbar-Button 'ffnen'
        button = new JButton(new ImageIcon(ICON_OPEN));
        button.setToolTipText(LABEL_OPEN);
        button.setActionCommand("OPEN");
        button.addActionListener(this);
        toolBar.add(button);

        // Toolbar-Button 'Speichern'
        button = new JButton(new ImageIcon(ICON_SAVE));
        button.setToolTipText(LABEL_SAVE);
        button.setActionCommand("SAVE");
        button.addActionListener(this);
        toolBar.add(button);

        toolBar.addSeparator();

        // Toolbar-Button 'Bearbeiten'
        button = new JButton(new ImageIcon(ICON_EDIT));
        button.setToolTipText(LABEL_EDIT);
        button.setActionCommand("EDIT");
        button.addActionListener(this);
        toolBar.add(button);

        toolBar.addSeparator();

        // Toolbar-Button 'ber'
        button = new JButton(new ImageIcon(ICON_ABOUT));
        button.setToolTipText(LABEL_ABOUT);
        button.setActionCommand("ABOUT");
        button.addActionListener(this);
        toolBar.add(button);
    }

    /* grafische Fehlermeldung */
    private void errorBox(String text) {
        debug("Error: " + text);
        JOptionPane.showMessageDialog(frame, text, "Fehler",
                JOptionPane.ERROR_MESSAGE);
    }

    /* Debug-Ausgabe */
    private void debug(String msg) {
        if (DEBUG) {
            System.out.println("DEBUG ::[ " + msg);
        }
    }
}